#define EXTERN
#include "../SRC/pglobal.h"
#include "../SRC/seqfilesm.h"
#include "../../GEFEL/xfile.h"

#include "input.h"
#include "output.h"
#include "globprep.h"
#include "elemhead.h"

#include <stdio.h>
#include <string.h>
#include "mpi.h"

int main (int argc,char *argv[])
{
  FILE *tmpf;
  XFILE *in;
  long err;
  descrip *d;

  fprintf (stderr,"\n\n *** PARMEFEL PREPROCESSOR ***\n");
  fprintf (stderr," -----------------------------\n");
  
  Mp  = new probdesc;
  Pmp = new pprobdesc;
  Gtm = new gtopology;
  Mt  = new mechtop;
  Mm  = new mechmat;
  Mc  = new mechcrsec;
  Mb  = new mechbclc;
  Outdm = new outdriverm;

  Bar2d = new barel2d;
  Beam2d = new beamel2d;
  Beam3d = new beamel3d;
  Sbeam = new soilbeam;
  Spring = new springel;
  Pelt = new planeelemlt;
  Peqt = new planeelemqt;
  Perlt = new planeelemrotlt;
  Pelq = new planeelemlq;
  Peqq = new planeelemqq;
  Perlq = new planeelemrotlq;
  Pesqt = new planeelemsubqt;
  Cct = new cctelem;
  Dkt = new dktelem;
  Dst = new dstelem;
  Q4pl = new q4plate;
  Asymlt = new axisymlt;
  Asymlq = new axisymlq;
  Asymqq = new axisymqq;
  Shtr = new shelltr;
  Shq = new shellq;
  Ltet = new lintet;
  Lhex = new linhex;
  Qhex = new quadhex;
  Spltr = new soilplatetr;
  Splq = new soilplateq;


  if (argc < 3){
    fprintf (stderr,"\n Wrong number of command line parameters.");
    fprintf (stderr,"\n Use : pmechprep input_file_name output_file name\n\n");
    delete Mp;  delete Mt;  delete Mm;  delete Mc;  delete Mb;  delete Pmp;
    return(1);
  }
  in = xfopen(argv[1],"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    delete Mp;  delete Mt;  delete Mm;  delete Mc;  delete Mb;  delete Pmp;
    return(2);
  }
  in->maxlnsize = 1024;
  in->warrning = 1;
  in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;

  d = new descrip;
  // reading line with topology file name
  xfscanf(in, "%a", d->t3df);
  // reading line with material database file name
  xfscanf(in, "%a", d->matf);
  // reading line with cross-section database file name
  xfscanf(in, "%a", d->crf);
  // reading line with topology file format indicator,
  xfscanf(in, "%ld", &d->t3d);
  d->seq = 0;
  // reading line with edge number indicator
  xfscanf(in, "%ld", &d->redgn);
  // reading problem description
  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "begin");
  in->kwdmode = ignore;

  //  creation of object of parallel solver
  Psol = new psolver (Nproc,Myrank);
  xfscanf (in,"%ld",&Ndom);
  Mp->read(in);
  Psol->read (in,Ndom,Mp->tstorsm,Mespr);

  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "end");
  in->kwdmode = ignore;
  

  if (Psol->tdd==layered_plate)
    d->seq = 1;


  err = input(in, d);
  if(err)
  {
    xfclose(in);
    fprintf (stderr,"\n\n");
    fprintf (stdout,"\n\n");
    return(err);
  }

  // reading output description
  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "begin");
  in->kwdmode = ignore;
  if (Mp->tprob == growing_mech_structure){
    Gtm->read_gf (in);
  }
  Outdm->read(in);
  xfclose(in);


  FILE *out = fopen(argv[2], "wt");
  if (out==NULL){
    fprintf (stderr,"\n Output file couldn't be opened.");
    fprintf (stderr,"\n Try it again!\n\n");
    fprintf (stderr,"\n\n");
    fprintf (stdout,"\n\n");
    return(7);
  }

  fprintf(stdout, "\nGeneral topology export . . .");
  Top->exporttop(Gtm);
  fprintf(stdout, " O.K.\n");

  fprintf(stdout, "\nProblem description output . . .");
  fprintf (out,"%ld\n",Ndom);
  Mp->print(out);
  Psol->print (out);
  fprintf(stdout, " O.K.\n");

  fprintf(stdout, "\nMechanical topology export . . .");
  Mt->alloc_prep (Top->nodes,Top->elements);
  fprintf(stdout, " O.K.\n");

  output(out, d);

  fprintf(stdout, "\nOutdriver description output . . .");
  Outdm->print(out);
  fprintf(stdout, " O.K.\n");

  fclose(out);

  fprintf (stderr,"\n\n");
  fprintf (stdout,"\n\n");
  delete d;
  //delete Mp;  delete Mt;  delete Gtm;  delete Mm;  delete Mc;  delete Mb;  delete Pmp;
  return(0);
}
