#include "pmefelinit.h"
#include "pglobal.h"
#include "xfile.h"

void pmefel_init (int argc,char *argv)
{
  long i;
  char name[1100];
  XFILE *in;

  //  creation of main objects
  Mp  = new probdesc;
  Pmp = new pprobdesc;
  Gtm = new gtopology;

  Mt  = new mechtop;
  Mm  = new mechmat;
  Mc  = new mechcrsec;
  Mb  = new mechbclc;
  
  Outdm = new outdriverm;
  
  /*
  if (Myrank==0){
    char bd;
    scanf ("%c",&bd);
  }
  MPI_Barrier (MPI_COMM_WORLD);
  */

  //  names of input files
  strcpy (name,argv);
  sprintf (&name[strlen (argv)],"%d.in",Myrank+1);
  in = xfopen (name,"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file is not open on processor %d.\n",Myrank);
    abort();
  }
  in->maxlnsize = 1000;
  in->warrning = 1;
  in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;

  //  number of subdomain
  xfscanf (in,"%ld",&Ndom);
  Ndom--;

  //  reading data describing the problem
  Mp->read (in);
  
  //  creation of object of parallel solver
  Psol = new psolver (Nproc,Myrank);
  
  //  reading data about parallel solver
  Psol->read (in,Ndom,Mp->tstorsm,Mespr);
  

  char buf[1001];
  strcpy (buf,argv);
  sprintf (&buf[strlen (argv)],"%d.log",Myrank+1);
  Out = fopen (buf,"w");
  if (Out==NULL){
    fprintf (stderr,"\n Log file is not open on processor %d.\n",Myrank);
  }

  
  //  correspondence between processors and subdomains
  Psol->procdomcorr ();
  
  
  if (Myrank==0){
    fprintf (stderr,"\n\n kontrola domproc \n");
    fprintf (Out,"\n\n kontrola domproc \n");
    for (long i=0;i<Nproc;i++){
      fprintf (Out,"\n %ld  %ld",i,Psol->domproc[i]);
      fprintf (stderr,"\n %ld  %ld",i,Psol->domproc[i]);
    }
  }
  


  //  input of node and element data
  Mt->read (in);
  
  //  initiation of parallel solver
  Psol->initiate (Gtm);
  
  //  reading of the array ltg (local-to-global correspondence)
  Psol->read_ltg (in,Gtm,Out);
  
  fprintf (Out,"\n Nproc  %d",Nproc);
  fprintf (Out,"\n Ndom   %d",Ndom);
  fprintf (Out,"\n Myrank %d",Myrank);
  

  //  code numbers generation
  Ndofm=Psol->ordering (Gtm,Out);
  



  //  kontrolni tisk
  //fprintf (Out,"\n\n kontrola rohovych uzlu \n");
  //for (long i=0;i<Mt->nn;i++){
  //fprintf (Out,"\n %4ld %4ld",i,Psol->ltg[i]);
  //}
  //fprintf (Out,"\n\n konec kontroly rohovych uzlu \n\n");
  


  // provizorni umisteni, po skonceni debuggovani vyhodit
  //Psol->lpp->ndof=Ndofm;
    
  if (Mespr==1)  fprintf (stdout,"\n number of DOFs  %ld",Ndofm);
  



  // ***********************************************
  // **  input of material models and parameters  **
  // ***********************************************

  //  reading of material models and parameters
  //  allocation of integation points
  //  definition of the number of material models (nm) on integration points
  //  definition of material types (tm) on integration points
  //  definition of number of material model (idm) on integration points
  Mm->read (in);
  //  definition of variables ncompstr on intergation points
  //  definition of variables ssst on intergation points
  Mm->init_ip_1 ();



  // **********************************************
  // **  input of cross section characteristics  **
  // **********************************************
  Mc->read (in);
  
  
  // ************************
  // **  input of loading  **
  // ************************
  Mb->read (in);

  Lsrs = new lhsrhs;
  Lsrs->alloc ();

  // ******************************************
  // **  definition of output from the code  **
  // ******************************************
  Outdm->read(in);
  
  

  //  names of output files

  strcpy (name,Outdm->outfn);
  sprintf (&name[strlen (Outdm->outfn)],"%d.out",Myrank+1);
  strcpy (Outdm->outfn, name);
  if (Outdm->gf != grfmt_no)
  {
    strcpy (name,Outdm->outgrfn);
    sprintf (&name[strlen (Outdm->outgrfn)],"%d",Myrank+1);
    strcpy (Outdm->outgrfn, name);
  }
  if (Outdm->ndiag > 0)
  {
    strcpy (name,Outdm->outdiagfn);
    sprintf (&name[strlen (Outdm->outdiagfn)],"%d.dat",Myrank+1);
    strcpy (Outdm->outdiagfn, name);
  }  
  
  //fprintf (stdout,"\n procesor %ld    vystup %s    grafika %s   diagram %s",Myrank,Outdm->outfn,Outdm->outgrfn,Outdm->outdiagfn);
  
  //print_init(-1, "wt");    
  //Outdm->outf = fopen(Outdm->outfn, "w");
  
  //  patek
  //Psol->nodesplit (Gtm,Outdm->outf);
  //Psol->nodesplit (Gtm,Out);
  


  Mm->stra.init(strain);  
  Mm->stre.init(stress);  


  //  zacatek TKo
  //  definition of nonlocal models and models for temperature
  Mm->init_ip_2 ();
  //  konec TKo

  
  //  allocation of arrays defined on nodes
  Mt->alloc_nodes_arrays ();

  //  number of components in arrays used in outdriver
  //  maximum number of components in array strain/stress on nodes and elements
  Mt->give_maxncompstr(Mm->max_ncompstrn, Mm->max_ncompstre);
  //  maximum number of components in array other on nodes and elements
  Mt->give_maxncompo(Mm->max_nncompo, Mm->max_encompo);

  //  prescribed displacements on elements investigation
  Mt->elemprescdisp ();

  //  prescribed temperatures on elements investigation
  if (Mp->tprob == linear_statics){
    Mt->elempresctemp (0);
  }
  //  reactions on elements investigation
  if (Mp->reactcomp==1){
    Mt->comreac ();
  }

  //  computation of initial values at integration points from initial nodal values
  if (Mb->ico > 0)
    Mb->inicipval();

  //  initiation and computation of eigenstrains
  if (Mp->temperature==1){
    Mm->alloceigstrains ();
    Mm->alloctempstrains ();
    Mm->alloceigstresses ();
    
    Mm->tempr = new double [Mm->tnip];
    Mm->inittempr = new double [Mm->tnip];
    
    for (i=0;i<Mm->tnip;i++){
      Mm->tempr[i]=0.0;
      Mm->inittempr[i]=0.0;
    }
    
  }

  

  fflush(stdout);
  fflush(stderr);
  
  Pmp->shift_indices ();
  
  xfclose (in);
  

}
