#include "pmtsolver.h"
#include "pglobal.h"
#include "seqfilesm.h"
#include "genfile.h"
#include <string.h>
#include "mpi.h"


/**
   function solves time dependent mechanical problem in parallel
   inertial forces are negligible
   
   JK, 24.7.2008
*/
void par_solve_timemech_prob ()
{
  long i,j,k,n,ini,lcid,nsts,nii;
  double end_time,zero,err,dt,dtmin,dtdef,norfa,norf;
  double *f,*fi,*fp,*fb,*fl,*dr,*r,*lhsb;
  char file[FNAMELEN+20];
  FILE *aux;
  char *path;
  char *name;
  char *suffix;
  

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  sprintf (file,"%s.bac",Outdm->outfn);
  aux = fopen (file,"w");
  
  //  load case id must be equal to zero
  lcid=0;
  
  //  number of degrees of freedom (rows of the matrix)
  n = Ndofm;
  //  computer zero (computer threshold)
  zero = Mp->zero;
  //  maximum number of iterations in inner loop
  ini = Mp->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->errnr;
  
  
  fp = new double [n];
  fb = new double [n];
  fl = new double [n];
  fi = new double [n];
  dr = new double [n];
  lhsb = new double [n];
  memset (fp,0,n*sizeof(double));
  memset (fb,0,n*sizeof(double));
  memset (fl,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  memset (lhsb,0,n*sizeof(double));

  //  nodal displacements
  r = Lsrs->give_lhs (lcid);
  //  prescribed nodal forces
  f= Lsrs->give_rhs (lcid);
  
  
  //  initial time
  Mp->time=Mp->timecon.starttime ();
  //  end time
  end_time = Mp->timecon.endtime ();
  //  initial time increment
  dt = Mp->timecon.initialtimeincr ();
  //  minimum time increment
  dtmin=Mp->timecon.dtmin;
  
  
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  //  konec provizoria
  
  
  nullv (fp,n);
  //  number of step
  i=0;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;
  

  print_init(-1, "wt",Pmp->fni,Pmp->fei);
  print_step(lcid, i, Mp->time, f);
  print_flush();



  // ***************************
  //   main iteration loop  ****
  // ***************************
  do{
    
    //new time increment
    Mp->time = Mp->timecon.newtime (dt);
    //  new step number
    i++;
    //  indicator of strain computation
    //  it means, strains at integration points have not been computed
    Mp->strainstate=0;
    //  indicator of stress computation
    //  it means, stresses at integration points have not been computed
    Mp->stressstate=0;
    //  indicator of computation of other array
    //  it means, stresses at integration points have not been computed
    Mp->otherstate=0;
    
    
    //  backup of attained nodal values
    for (j=0;j<n;j++){
      lhsb[j]=r[j];
    }
    
    if (Mespr==1)
      fprintf (stdout,"\n time %e",Mp->time);
    fprintf (Out,"\n time %e",Mp->time);
    
    //    fprintf (Out,"\n\n\n\n time %e",Mp->time);

    
    //  prescribed nodal forces
    mefel_right_hand_side (lcid,f,fl);

    Mb->comp_sum (f);
    
    //  increments of prescribed forces
    incr_internal_forces (lcid,fi);
    
    //  right hand side assembling
    for (j=0;j<n;j++){
      fb[j]=f[j]-fp[j]+fi[j];
      fp[j]=f[j];
    }
    
    //  assembling of tangent stiffness matrix
    stiffness_matrix (lcid);

    //  solution of K(r).v=F
    //  solution of equation system
    Psol->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);

    //  new displacement vector
    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }
    
    //  computation of internal forces
    internal_forces (lcid,fi);
    
    
    norfa=Psol->pss (f,Out);
    
    for (j=0;j<n;j++){
      fb[j] = fl[j] - fi[j];
    }
    //  norm of vector of unbalanced forces
    norf=Psol->pss(fb,Out);
    
    j = 0;
    if (norfa > 0.0)
      norf /= norfa;
    if (Psol->compare_on_master(norf, err) <= 0) // norf <= err
    {
      //  number of successful time steps
      nsts++;
      //  no inner iteration
      nii=1;
    }
    else
      nii=0;
    
    
    
    if (nii==0){
      
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      for (j=0;j<ini;j++){
	
        //  assembling of tangent stiffness matrix
        //stiffness_matrix (lcid);
	//  solution of K(r).v=F
	//Smat->solve_system (Gtm,dr,fb);
	Psol->par_linear_solver (Gtm,Smat,dr,fb,Out,Mespr);
	
	for (k=0;k<n;k++)
	  r[k]+=dr[k];
	
	//  computation of internal forces
	internal_forces (lcid,fi);
	
	//  vector of unbalanced forces
	for (k=0;k<n;k++)
	  fb[k]=fl[k]-fi[k];
	
	//  norm of vector of unbalanced forces
	norf=Psol->pss(fb,Out);
	
        if (norfa > 0.0)
	  norf /= norfa;
	if ((Myrank==0) && (Mespr==1))
	  fprintf (stdout,"\n Time increment %lf   inner loop j %ld     norf=%e",Mp->time,j,norf);
        if (Psol->compare_on_master(norf, err) <= 0) // norf <= err
        {
          //  number of successful time steps
          nsts++;
          //  no inner iteration
          nii=1;
        }
        else
          nii=0;
	
	if (nii==1)
	  break;
      }
    }
    
    if (nii==0){
      //  backup is used for actual values
      for (j=0;j<n;j++){
	r[j]=lhsb[j];
      }
      //  reduction of the time increment because
      //  inner loop was not able to enforce equilibrium
      dt/=2.0;
      Mp->timecon.oldtime ();
      i--;
      
      if ((Myrank==0) && (Mespr==1))
        fprintf (stdout,"\n time increment is reduced because the inner loop was not able to enforce equilibrium");
      if (dt<dtmin){
	fprintf (stderr,"\n\n time increment is less than minimum time increment");
	fprintf (stderr,"\n computation fails (file %s, line %d)\n",__FILE__,__LINE__);
        abort();
      }
      
    }
    else{
      dtdef = Mp->timecon.actualforwtimeincr ();
      if (nsts==2){
	dt*=2.0;
	nsts=0;
	if ((Myrank==0) && (Mespr==1))
          fprintf (stdout,"\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
      }
      if (dt>dtdef)
	dt=dtdef;
      
      Mm->updateipval();
      // equilibrium state was reached
      compute_req_val (lcid);
      print_step(lcid, i, Mp->time, f);
      print_flush();
      
    }
    
  }while(Mp->time<end_time);
  
  
  print_close ();
  
  fclose (aux);

  delete [] dr;  delete [] fi;  delete [] fb;  delete [] fp;  delete [] fl;  delete [] lhsb;
}



void par_solve_timemech_prob_old ()
{
  long i,j,n,lcid;
  double dt,end_time,zero,*f,*fi,*fp,*dr,*lhs,*rhs;
  char file[FNAMELEN+20];
  FILE *aux;
  char *path;
  char *name;
  char *suffix;
  

  Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  sprintf (file,"%s.bac",Outdm->outfn);
  aux = fopen (file,"w");


  lcid=0;
  
  //  number of rows of the matrix
  n = Ndofm;
  //  computer zero
  zero = Mp->zero;
  
  fp = new double [n];
  f  = new double [n];
  fi = new double [n];
  dr = new double [n];
  memset (fp,0,n*sizeof(double));
  memset (f,0,n*sizeof(double));
  memset (fi,0,n*sizeof(double));
  memset (dr,0,n*sizeof(double));
  
  //  initialization phase
  lhs = Lsrs->give_lhs (lcid);
  rhs = Lsrs->give_rhs (lcid);
  
  
  //  initial time
  Mp->time=Mp->timecon.starttime ();
  //  end time
  end_time = Mp->timecon.endtime ();
  
  //  allocation of backup arrays for state variables in integration points
  //Mm->alloc_backup ();
  
  //  provizorni umisteni, bude v mefelinit
  Gtm->comp_domain_sizes ();
  Mb->alloc_sumcomp ();
  //  konec provizoria
  
  
  nullv (fp,n);
  i=0;
  

  print_init(-1, "wt",Pmp->fni,Pmp->fei);
  print_step(lcid, i, Mp->time, f);
  print_flush();



  // ***************************
  //   main iteration loop  ****
  // ***************************

  do{
    
    //  update integration points indicators
    Mm->updateipval();
    

    //if (Myrank==0){
    //fprintf (stderr,"\n fajmanova matice %lf %lf %lf %lf",Mm->crbaz[0].apom[0][0],Mm->crbaz[0].apom[1][1],
    //Mm->crbaz[0].apom[2][2],Mm->crbaz[0].apom[3][3]);
    //}
    
    //if (Myrank==0)
    //fprintf (stdout,"\n proc %ld  po updateipval      time %lf",Myrank,Mp->time);
    
    //fprintf (stdout,"\n time %e",Mp->time);
    //fprintf (Out,"\n\n\n\n time %e",Mp->time);
    

    //  prescribed nodal forces
    mefel_right_hand_side (lcid,f);

    //fprintf (Out,"\n\n kontrola prave strany v case %e",Mp->time);
    //for (j=0;j<n;j++){
    //fprintf (Out,"\n f %5ld   %le",j,f[j]);
    //}
    
    Mb->comp_sum (f);
    

    //  unbalanced forces
    Mp->phase=1;
    internal_forces (lcid,fi);
    
    //  right hand side assembling
    for (j=0;j<n;j++){
      rhs[j]=f[j]-fp[j]+fi[j];
      fp[j]=f[j];
    }
    
    
    //  assembling of tangent stiffness matrix
    stiffness_matrix (lcid);
    
    //if (Myrank==0){
    //fprintf (stdout,"\n Smat %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
    //Smat->sky->a[0],Smat->sky->a[1],Smat->sky->a[2],Smat->sky->a[3],Smat->sky->a[4],
    //Smat->sky->a[5],Smat->sky->a[6],Smat->sky->a[7],Smat->sky->a[8],Smat->sky->a[9]);
    //}
    
    
    //  solution of K(r).v=F
    //  solution of equation system
    Psol->par_linear_solver (Gtm,Smat,dr,rhs,Out,Mespr);
    
    
    //fprintf (stdout,"\n dr[0] %lf",dr[0]);
    
    
    //  new displacement vector
    for (j=0;j<n;j++){
      lhs[j]+=dr[j];
    }
    
    //  computation of internal forces
    Mp->phase=2;
    internal_forces (lcid,fi);
    
    compute_req_val (lcid);
    

    //  time increment
    Mp->time = Mp->timecon.newtime ();
    i++;
    

    if (Myrank==0)
      fprintf (stdout,"\n step %ld   time %lf",i,Mp->time);
    

    if (Mp->timecon.isitimptime ()==1){
      mtsolver_save (lhs,fp,i,Mp->time,dt,n);
    }




    print_step(lcid, i, Mp->time, f);
    print_flush();


    /*
      if ((Outdm->no.dstep.presence_id(i, Mp->time)) || 
      (Outdm->eo.dstep.presence_id(i, Mp->time)))
      {
      //long jk=i;
      //jk=jk/20;
      //jk=jk*20;
      //if (jk==i){
      print_init(i, "wt",Pmp->fni,Pmp->fei);
      print_step(lcid, i, Mp->time, f);
      //print_flush();(eo.dstep.presence_id(istep, time)
      print_close();
      //}
      }
    */
    
    
    
  }while(Mp->time<end_time);
  
  
  print_close ();
  
  fclose (aux);

  delete [] dr;  delete [] fi;  delete [] fp;  delete [] f;

}

